'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _objectAssign = require('object-assign');

var _objectAssign2 = _interopRequireDefault(_objectAssign);

var _isPlainObject = require('is-plain-object');

var _isPlainObject2 = _interopRequireDefault(_isPlainObject);

var _isPromise = require('is-promise');

var _isPromise2 = _interopRequireDefault(_isPromise);

var _container = require('./container');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var _setStore = function _setStore(store, obj) {
  var cache = (0, _container.get)(store);
  if (cache) {
    (0, _objectAssign2.default)(cache.store, obj);
    cache.list.forEach(function (update) {
      return update();
    });
  }
};

var transformStore = function transformStore(store) {
  var map = {};
  var event = {
    store: map,
    setStore: function setStore(obj) {
      return _setStore(store, obj);
    }
  };
  var mergeStore = _extends({
    setStore: function setStore(obj) {
      return obj;
    }
  }, store);

  var _loop = function _loop(key) {
    var item = mergeStore[key];
    map[key] = typeof item === 'function' ? function () {
      var obj = item.apply(undefined, Array.prototype.slice.call(arguments).concat([event]));
      if ((0, _isPromise2.default)(obj)) {
        obj.then(function (value) {
          return event.setStore(value);
        });
      } else if ((0, _isPlainObject2.default)(obj)) {
        event.setStore(obj);
      }
    } : item;
  };

  for (var key in mergeStore) {
    _loop(key);
  }
  return map;
};

var createStore = function createStore(store, update) {
  var map = {};
  for (var name in store) {
    var obj = store[name];
    if (Object.freeze) {
      Object.freeze(obj);
    }
    var cache = (0, _container.get)(obj);
    if (cache) {
      if (update) {
        cache.list.push(update);
      }
    } else {
      cache = {
        list: update ? [update] : [],
        store: transformStore(obj)
      };
      (0, _container.set)(obj, cache);
    }
    map[name] = cache.store;
  }
  return map;
};

var delStore = function delStore(store, update) {
  for (var name in store) {
    var obj = store[name];
    var cache = (0, _container.get)(obj);
    if (cache) {
      var list = cache.list.filter(function (item) {
        return item !== update;
      });
      if (list.length > 0) {
        cache.list = list;
      } else {
        (0, _container.del)(cache);
      }
    }
  }
};

exports.default = { createStore: createStore, setStore: _setStore, delStore: delStore, transformStore: transformStore };
module.exports = exports['default'];